@@ -199,23 +199,10 @@ BC_R(store_chartgroup_type)(ExcelChartReadState *s, char const *t)
 	g_return_val_if_fail (fmt == NULL, NULL);
 
 	fmt = xmlNewChild (s->xml.currentChartGroup, s->xml.ns, "Type", NULL);
-	return xmlNewChild (fmt, s->xml.ns, t, NULL);
+	xmlSetProp (fmt, "name", t);
+	return fmt;
 }
 
-#if 0
-static xmlNode *
-BC_R(get_chartgroup_type)(ExcelChartReadState *s, char const *t)
-{       
-	xmlNode *fmt;
-
-	g_return_val_if_fail (s->xml.currentChartGroup != NULL, NULL);
-
-	fmt = e_xml_get_child_by_name (s->xml.currentChartGroup, "Type");
-	g_return_val_if_fail (fmt != NULL, NULL);
-	return e_xml_get_child_by_name (fmt, t);
-}
-#endif
-
 /****************************************************************************/
 
 static gboolean
@@ -680,25 +667,26 @@ BC_R(bar)(ExcelChartHandler const *handle,
 {
 	guint16 const flags = MS_OLE_GET_GUINT16 (q->data+4);
 
-	xmlNode * fmt = BC_R(store_chartgroup_type)(s, "Bar");
+	xmlNode *tmp, *fmt = BC_R(store_chartgroup_type)(s, "Bar");
 
 	g_return_val_if_fail (fmt != NULL, TRUE);
 
-	e_xml_set_bool_prop_by_name (fmt, "horizontal", 
-				     (flags & 0x01) ? TRUE : FALSE);
+	/* Always set this it makes things clearer */
+	xmlNewChild (fmt, fmt->ns, "horizontal",
+		(flags & 0x01) ? "true" : "false");
 
 	if (flags & 0x04)
-		e_xml_set_bool_prop_by_name (fmt, "as_percentage", TRUE);
+		xmlNewChild (fmt, fmt->ns, "as_percentage", NULL);
 	else if (flags & 0x02)
-		e_xml_set_bool_prop_by_name (fmt, "stacked", TRUE);
+		xmlNewChild (fmt, fmt->ns, "stacked", NULL);
 
 	if (s->container.ver >= MS_BIFF_V8 && (flags & 0x08))
-		e_xml_set_bool_prop_by_name (fmt, "in_3d", TRUE);
+		xmlNewChild (fmt, fmt->ns, "in_3d", NULL);
 
-	xml_node_set_int (fmt, "percentage_space_between_items",
-			  MS_OLE_GET_GUINT16 (q->data));
-	xml_node_set_int (fmt, "percentage_space_between_groups",
-			  MS_OLE_GET_GUINT16 (q->data+2));
+	tmp = xmlNewChild (fmt, fmt->ns, "percentage_space_between_items", NULL);
+	xml_node_set_int (tmp, NULL, MS_OLE_GET_GUINT16 (q->data));
+	tmp = xmlNewChild (fmt, fmt->ns, "percentage_space_between_groups", NULL);
+	xml_node_set_int (tmp, NULL, MS_OLE_GET_GUINT16 (q->data+2));
 
 	return FALSE;
 }
@@ -1021,8 +1009,10 @@ static gboolean
 BC_R(dropbar)(ExcelChartHandler const *handle,
 	      ExcelChartReadState *s, BiffQuery *q)
 {
-	guint16 const width = MS_OLE_GET_GUINT16 (q->data);	/* 0-100 */
-	g_return_val_if_fail (width <= 100, FALSE);
+	/* NOTE : The docs lie.  values > 100 seem legal.  My guess based on
+	 * the ui is 500.
+	guint16 const width = MS_OLE_GET_GUINT16 (q->data);
+	 */
 	return FALSE;
 }
 
@@ -1165,18 +1155,18 @@ BC_R(legend)(ExcelChartHandler const *handle,
 	guint16 const flags = MS_OLE_GET_GUINT16  (q->data+18);
 #endif
 	guint16 const position = MS_OLE_GET_GUINT8 (q->data+16);
-	char const *position_txt = "right";
+	char const *position_txt = "east";
 	xmlNode *legend;
 
 	switch (position) {
-	case 0: position_txt = "bottom"; break;
+	case 0: position_txt = "south"; break;
 	case 1: break; /* What is corner ? */
-	case 2: position_txt = "top";	break;
-	case 3: break; /* right */
-	case 4: position_txt = "left";	break;
-	case 7: break; /* treat floating legends as being on right */
+	case 2: position_txt = "north";	break;
+	case 3: break; /* east */
+	case 4: position_txt = "west";	break;
+	case 7: break; /* treat floating legends as being on east */
 	default :
-		g_warning ("Unknown legend position (%d), assuming right.",
+		g_warning ("Unknown legend position (%d), assuming east.",
 			   position);
 	};
 
@@ -1229,17 +1219,17 @@ BC_R(line)(ExcelChartHandler const *handle,
 {
 	guint16 const flags = MS_OLE_GET_GUINT16 (q->data);
 
-	xmlNode * fmt = BC_R(store_chartgroup_type)(s, "Line");
+	xmlNode *fmt = BC_R(store_chartgroup_type)(s, "Line");
 
 	g_return_val_if_fail (fmt != NULL, TRUE);
 
 	if (flags & 0x02)
-		e_xml_set_bool_prop_by_name (fmt, "as_percentage", TRUE);
+		xmlNewChild (fmt, fmt->ns, "as_percentage", NULL);
 	else if (flags & 0x01)
-		e_xml_set_bool_prop_by_name (fmt, "stacked", TRUE);
+		xmlNewChild (fmt, fmt->ns, "stacked", NULL);
 
 	if (s->container.ver >= MS_BIFF_V8 && (flags & 0x04))
-		e_xml_set_bool_prop_by_name (fmt, "in_3d", TRUE);
+		xmlNewChild (fmt, fmt->ns, "in_3d", NULL);
 
 	return FALSE;
 }
@@ -1312,7 +1302,7 @@ BC_R(lineformat)(ExcelChartHandler const *handle,
 
 	/* Applies to frames too */
 	if (s->xml.dataFormat != NULL) {
-		line = e_xml_get_child_by_name (s->xml.dataFormat, "Area");
+		line = e_xml_get_child_by_name (s->xml.dataFormat, "Line");
 		if (line == NULL)
 			line = xmlNewChild (s->xml.dataFormat, s->xml.ns,
 					    "Line", NULL);
@@ -1460,23 +1450,30 @@ BC_R(pie)(ExcelChartHandler const *handle,
 	  ExcelChartReadState *s, BiffQuery *q)
 {
 	double radians;
-	xmlNode *fmt = BC_R(store_chartgroup_type)(s, "Pie");
+	xmlNode *tmp, *fmt = BC_R(store_chartgroup_type)(s, "Pie");
+	guint16 const percent_diam = MS_OLE_GET_GUINT16 (q->data+2); /* 0-100 */
+
+	/* This is for the whole pie */
+	if (percent_diam > 0) {
+		xmlNode *tmp = xmlNewChild (fmt, fmt->ns,
+			"separation_percent_of_radius", NULL);
+		xml_node_set_int (tmp, NULL, percent_diam);
+	}
 
-	/* XL counts 0 at a different position */
 	radians = MS_OLE_GET_GUINT16 (q->data);
-	radians = (radians * 2. * M_PI / 360 - M_PI / 2.);
-	xml_node_set_double (fmt, "radians_of_first_pie",
-		radians, -1); 
+	radians = (radians * 2. * M_PI / 360.);
+	tmp = xmlNewChild (fmt, fmt->ns, "radians_of_first_pie", NULL);
+	xml_node_set_double (fmt, NULL, radians, -1); 
 
-	xml_node_set_int (fmt, "hole_percentage_of_diameter", 
-		MS_OLE_GET_GUINT16 (q->data+2));
 	if (s->container.ver >= MS_BIFF_V8) {
 		guint16 const flags = MS_OLE_GET_GUINT16 (q->data+4);
 
 		if (flags & 0x1)
-			e_xml_set_bool_prop_by_name (fmt, "in_3d", TRUE);
+			xmlNewChild (fmt, fmt->ns, "in_3d", NULL);
+#if 0
 		if (flags & 0x2)
 			e_xml_set_bool_prop_by_name (fmt, "leader_lines", TRUE);
+#endif
 	}
 
 	return FALSE;
@@ -1505,8 +1502,12 @@ BC_R(pieformat)(ExcelChartHandler const *handle,
 	if (pie == NULL)
 		pie = xmlNewChild (s->xml.dataFormat, s->xml.ns, "Pie", NULL);
 
-	e_xml_set_integer_prop_by_name (pie,
-		"separation_percent_of_diameter", percent_diam);
+	/* This is for individual slices */
+	if (percent_diam > 0) {
+		xmlNode *tmp = xmlNewChild (pie, pie->ns,
+			"separation_percent_of_radius", NULL);
+		xml_node_set_int (tmp, NULL, percent_diam);
+	}
 
 	d (2, printf ("Pie slice is %hu %% of diam from center\n", percent_diam););
 	return FALSE;
@@ -1639,7 +1640,7 @@ static gboolean
 BC_R(scatter)(ExcelChartHandler const *handle,
 	      ExcelChartReadState *s, BiffQuery *q)
 {
-	xmlNode * fmt = BC_R(store_chartgroup_type)(s, "Scatter");
+	xmlNode *fmt = BC_R(store_chartgroup_type)(s, "Scatter");
 
 	g_return_val_if_fail (fmt != NULL, TRUE);
 
@@ -1651,17 +1652,20 @@ BC_R(scatter)(ExcelChartHandler const *handle,
 			guint16 const size_type = MS_OLE_GET_GUINT16 (q->data+2);
 			e_xml_set_bool_prop_by_name (fmt, "has_bubbles", TRUE);
 			if (!(flags & 0x02))
-				e_xml_set_bool_prop_by_name (fmt, "hide_negatives", TRUE);
+				xmlNewChild (fmt, fmt->ns, "hide_negatives", NULL);
 			if (flags & 0x04)
-				e_xml_set_bool_prop_by_name (fmt, "in_3d", TRUE);
+				xmlNewChild (fmt, fmt->ns, "in_3d", NULL);
 
+#if 0
+			/* huh ? */
 			xml_node_set_int (fmt, "percentage_largest_tochart",
 					  MS_OLE_GET_GUINT16 (q->data));
-			e_xml_set_bool_prop_by_name (fmt,
-						     (size_type == 2)
-						     ? "bubble_sized_as_width"
-						     : "bubble_sized_as_area",
-						     TRUE);
+#endif
+			xmlNewChild (fmt, fmt->ns,
+				     (size_type == 2)
+				     ? "bubble_sized_as_width"
+				     : "bubble_sized_as_area",
+				     NULL);
 		}
 	}
 
@@ -1719,13 +1723,18 @@ BC_R(vector_details)(ExcelChartReadState *s, BiffQuery *q, ExcelChartSeries *ser
 	guint16 e_type = MS_OLE_GET_GUINT16 (q->data + type_offset);
 
 	switch (e_type) {
-	case 0 : type = GNM_VECTOR_DATE; break;
-	case 1 : type = GNM_VECTOR_SCALAR; break;
-	case 2 :
-		g_warning ("Unsupported vector type 'sequences', converting to scalar");
-		type = GNM_VECTOR_SCALAR; break;
+	case 0 : g_warning ("Unsupported vector type 'date', converting to scalar");
+		 type = GNM_VECTOR_SCALAR;
+		 break;
 
-	case 3 : type = GNM_VECTOR_STRING; break;
+	case 1 : type = GNM_VECTOR_SCALAR;
+		 break;
+
+	case 2 : g_warning ("Unsupported vector type 'sequences', converting to scalar");
+		 type = GNM_VECTOR_SCALAR;
+		 break;
+	case 3 : type = GNM_VECTOR_STRING;
+		 break;
 
 	default :
 		g_warning ("Unsupported vector type '%d', converting to scalar", e_type);
@@ -2354,6 +2363,12 @@ chart_parse_expr  (MSContainer *container, guint8 const *data, int length)
 	return NULL;
 }
 
+static StyleFormat *
+chart_get_fmt (MSContainer const *container, guint16 indx)
+{
+	return ms_container_get_fmt (container->parent_container, indx);
+}
+
 gboolean
 ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver, GtkObject *graph)
 {
@@ -2361,7 +2376,8 @@ ms_excel_chart (BiffQuery *q, MSContainer *container, MsBiffVersion ver, GtkObje
 		chart_realize_obj,
 		chart_create_obj,
 		chart_parse_expr,
-		NULL, NULL
+		NULL,
+		chart_get_fmt
 	};
 	int const num_handler = sizeof(chart_biff_handler) /
 		sizeof(ExcelChartHandler *);
