@@ -21,8 +21,8 @@
  * USA
  */
 
-#include "config.h"
-#include <gnome.h>
+#include <config.h>
+#include "xml-io-version.h"
 #include "gnumeric.h"
 #include "io-context.h"
 #include "plugin.h"
@@ -35,12 +35,14 @@
 #include "style.h"
 #include "style-border.h"
 #include "style-color.h"
+#include "style-condition.h"
 #include "format.h"
 #include "cell.h"
 #include "position.h"
 #include "expr.h"
 #include "expr-name.h"
 #include "print-info.h"
+#include "validation.h"
 #include "value.h"
 #include "selection.h"
 #include "command-context.h"
@@ -50,6 +52,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <gnome.h>
 #include <gnome-xml/tree.h>
 #include <gnome-xml/parser.h>
 #include <gnome-xml/parserInternals.h>
@@ -143,7 +146,6 @@ static gboolean
 xml_sax_attr_cellpos (CHAR const * const *attrs, char const *name, CellPos *val)
 {
 	CellPos tmp;
-	int dummy;
 
 	g_return_val_if_fail (attrs != NULL, FALSE);
 	g_return_val_if_fail (attrs[0] != NULL, FALSE);
@@ -152,7 +154,7 @@ xml_sax_attr_cellpos (CHAR const * const *attrs, char const *name, CellPos *val)
 	if (strcmp (attrs[0], name))
 		return FALSE;
 
-	if (!parse_cell_name (attrs[1], &tmp.col, &tmp.row, TRUE, &dummy)) {
+	if (!parse_cell_name (attrs[1], &tmp.col, &tmp.row, TRUE, NULL)) {
 		g_warning ("Invalid attribute '%s', expected cellpos, received '%s'",
 			   name, attrs[1]);
 		return FALSE;
@@ -239,6 +241,7 @@ STATE_WB,
                 STATE_NAMES_NAME,
                         STATE_NAMES_NAME_NAME,
                         STATE_NAMES_NAME_VALUE,
+                        STATE_NAMES_NAME_POSITION,
 	STATE_WB_GEOMETRY,
 	STATE_WB_SHEETS,
 		STATE_SHEET,
@@ -250,6 +253,7 @@ STATE_WB,
 				STATE_SHEET_NAMES_NAME,
 					STATE_SHEET_NAMES_NAME_NAME,
 					STATE_SHEET_NAMES_NAME_VALUE,
+					STATE_SHEET_NAMES_NAME_POSITION,
 			STATE_SHEET_PRINTINFO,
                                 STATE_PRINT_MARGINS,
 					STATE_PRINT_MARGIN_TOP,
@@ -258,6 +262,7 @@ STATE_WB,
 					STATE_PRINT_MARGIN_RIGHT,
 					STATE_PRINT_MARGIN_HEADER,
 					STATE_PRINT_MARGIN_FOOTER,
+                                STATE_PRINT_SCALE,
 				STATE_PRINT_VCENTER,
 				STATE_PRINT_HCENTER,
 				STATE_PRINT_GRID,
@@ -283,6 +288,12 @@ STATE_WB,
 							STATE_BORDER_RIGHT,
 							STATE_BORDER_DIAG,
 							STATE_BORDER_REV_DIAG,
+						STATE_STYLE_VALIDATION,
+							STATE_STYLE_CONDITION_CHAIN,
+								STATE_STYLE_CONDITION,
+									STATE_STYLE_CONDITION_EXPR,
+									STATE_STYLE_CONDITION_CONSTRAINT,
+									STATE_STYLE_CONDITION_FLAGS,
 			STATE_SHEET_COLS,
 				STATE_COL,
 			STATE_SHEET_ROWS,
@@ -334,6 +345,7 @@ static char const * const xmlSax_state_names[] =
                 "gmr:Name",
                         "gmr:name",
                         "gmr:value",
+                        "gmr:position",
 	"gmr:Geometry",
 	"gmr:Sheets",
 		"gmr:Sheet",
@@ -345,6 +357,7 @@ static char const * const xmlSax_state_names[] =
 				"gmr:Name",
 					"gmr:name",
 					"gmr:value",
+					"gmr:position",
 			"gmr:PrintInformation",
 				"gmr:Margins",
 					"gmr:top",
@@ -353,6 +366,7 @@ static char const * const xmlSax_state_names[] =
 					"gmr:right",
 					"gmr:header",
 					"gmr:footer",
+				"gmr:Scale",
 				"gmr:vcenter",
 				"gmr:hcenter",
 				"gmr:grid",
@@ -378,6 +392,13 @@ static char const * const xmlSax_state_names[] =
 							"gmr:Right",
 							"gmr:Diagonal",
 							"gmr:Rev-Diagonal",
+						"gmr:Validation",
+							"gmr:StyleConditionChain",
+								"gmr:StyleCondition",
+									"gmr:Expr",
+									"gmr:Constraint",
+									"gmr:Flags",
+
 			"gmr:Cols",
 				"gmr:ColInfo",
 
@@ -405,21 +426,6 @@ static char const * const xmlSax_state_names[] =
 NULL
 };
 
-typedef enum
-{
-    GNUM_XML_UNKNOWN = 0,
-    GNUM_XML_V1,
-    GNUM_XML_V2,
-    GNUM_XML_V3,	/* >= 0.52 */
-    GNUM_XML_V4,	/* >= 0.57 */
-    GNUM_XML_V5,	/* >= 0.58 */
-    GNUM_XML_V6,	/* >= 0.62 */
-    GNUM_XML_V7,        /* >= 0.66 */
-    GNUM_XML_V8,        /* >= 0.68 */
-
-    /* NOTE : Keep this up to date */
-    GNUM_XML_LATEST = GNUM_XML_V8
-} GnumericXMLVersion;
 typedef struct _XMLSaxParseState
 {
 	xmlSaxState state;
@@ -446,7 +452,23 @@ typedef struct _XMLSaxParseState
 	struct {
 		char *name;
 		char *value;
+		char *position;
 	} name;
+
+	struct {
+		ValidationStyle  vs;
+		char            *title;
+		char            *msg;
+	} validation;
+
+	struct {
+		StyleConditionType type;
+		StyleConditionBool next_op;
+		StyleConditionBool prev_next_op;
+
+		StyleCondition *scf;
+		StyleCondition *scl;			
+	} condition;
 	
 	gboolean  style_range_init;
 	Range	  style_range;
@@ -827,6 +849,30 @@ xml_sax_print_margins (XMLSaxParseState *state, CHAR const **attrs)
 	}
 }
 
+static void
+xml_sax_print_scale (XMLSaxParseState *state, CHAR const **attrs)
+{
+	PrintInformation *pi;
+	double percentage;
+	int cols, rows;
+
+	g_return_if_fail (state->sheet != NULL);
+	g_return_if_fail (state->sheet->print_info != NULL);
+
+	pi = state->sheet->print_info;
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (!strcmp (attrs[0], "type"))
+			pi->scaling.type = strcmp (attrs[1], "percentage")
+				? SIZE_FIT : PERCENTAGE;
+		else if (xml_sax_attr_double (attrs, "percentage", &percentage))
+			pi->scaling.percentage = percentage;
+		else if (xml_sax_attr_int (attrs, "cols", &cols))
+			pi->scaling.dim.cols = cols;
+		else if (xml_sax_attr_int (attrs, "rows", &rows))
+			pi->scaling.dim.rows = rows;
+	}
+}
+
 static void
 xml_sax_selection_range (XMLSaxParseState *state, CHAR const **attrs)
 {
@@ -1125,6 +1171,131 @@ xml_sax_styleregion_font_end (XMLSaxParseState *state)
 	}
 }
 
+static void
+xml_sax_validation (XMLSaxParseState *state, CHAR const **attrs)
+{
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (xml_sax_attr_int (attrs, "Style", (int *) &state->validation.vs)) {
+		} else if (!strcmp (attrs[0], "Title")) {
+			state->validation.title = g_strdup (attrs[1]);
+		} else if (!strcmp (attrs[0], "Message")) {
+			state->validation.msg = g_strdup (attrs[1]);			
+		} else
+			xml_sax_unknown_attr (state, attrs, "StyleCondition");
+	}
+	state->condition.scf = NULL;
+	state->condition.scl = NULL;
+}
+
+static void
+xml_sax_validation_end (XMLSaxParseState *state)
+{
+	Validation *v;
+
+	g_return_if_fail (state->style != NULL);
+
+	v = validation_new (state->validation.vs, state->validation.title,
+			    state->validation.msg, state->condition.scf);
+	mstyle_set_validation (state->style, v);
+
+	if (state->validation.title) {
+		g_free (state->validation.title);
+		state->validation.title = NULL;
+	}
+	if (state->validation.msg) {
+		g_free (state->validation.msg);
+		state->validation.msg = NULL;
+	}
+	state->condition.scf = NULL;
+	state->condition.scl = NULL;
+}
+
+static void
+xml_sax_style_condition (XMLSaxParseState *state, CHAR const **attrs)
+{
+	state->condition.prev_next_op = state->condition.next_op;
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (xml_sax_attr_int (attrs, "Type", (int *) &state->condition.type));
+		else if (xml_sax_attr_int (attrs, "NextOperator", (int *) &state->condition.next_op));
+		else
+			xml_sax_unknown_attr (state, attrs, "StyleCondition");
+	}
+}
+
+static void
+xml_sax_style_condition_chain (XMLSaxParseState *state, StyleCondition *sc)
+{
+	if (state->condition.scl) {
+		style_condition_chain (state->condition.scl, state->condition.prev_next_op, sc);
+		state->condition.scl = sc;
+	} else {
+		state->condition.scl = sc;
+		state->condition.scf = sc;
+	}
+}
+
+static void
+xml_sax_style_condition_expr (XMLSaxParseState *state, CHAR const **attrs)
+{
+	StyleConditionOperator op = -1;
+	StyleCondition *sc;
+	char const *cexpr = NULL;
+	ExprTree *expr = NULL;
+	
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (xml_sax_attr_int (attrs, "Operator", (int *) &op));
+		else if (!strcmp (attrs[0], "Expression"))
+			 cexpr = attrs[1];
+		else
+			xml_sax_unknown_attr (state, attrs, "StyleConditionExpr");
+	}
+
+	if (cexpr) {
+		ParsePos pos, *pp;
+		
+		pp = parse_pos_init (&pos, state->wb, state->sheet, 0, 0);
+		if ((expr = expr_parse_string (cexpr, pp, NULL, NULL)) == NULL)
+			g_warning ("XML-IO: empty/missing expression in StyleCondition");
+	} else
+		g_warning ("StyleConditionExpression without Expression!");
+
+	sc = style_condition_new_expr (state->sheet, op, expr);
+	expr_tree_unref (expr);
+	xml_sax_style_condition_chain (state, sc);
+}
+
+static void
+xml_sax_style_condition_constraint (XMLSaxParseState *state, CHAR const **attrs)
+{
+	StyleCondition *sc;
+	StyleConditionConstraint constraint = -1;
+	
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (xml_sax_attr_int (attrs, "Value", (int *) &constraint));
+		else
+			xml_sax_unknown_attr (state, attrs, "StyleConditionConstraint");
+	}
+
+	sc = style_condition_new_constraint (constraint);
+	xml_sax_style_condition_chain (state, sc);
+}
+
+static void
+xml_sax_style_condition_flags (XMLSaxParseState *state, CHAR const **attrs)
+{
+	StyleCondition *sc;
+	StyleConditionFlags flags;
+
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (xml_sax_attr_int (attrs, "Value", (int *) &flags));
+		else
+			xml_sax_unknown_attr (state, attrs, "StyleConditionFlags");
+	}
+
+	sc = style_condition_new_flags (flags);
+	xml_sax_style_condition_chain (state, sc);	
+}
+
 static void
 xml_sax_style_region_borders (XMLSaxParseState *state, CHAR const **attrs)
 {
@@ -1387,8 +1558,17 @@ xml_sax_finish_parse_wb_names_name (XMLSaxParseState *state)
 	
 	if (state->version >= GNUM_XML_V7) {
 		ParseError  perr;
+		ParsePos    pos;
 		
-		if (!expr_name_create (state->wb, NULL, state->name.name,
+		parse_pos_init (&pos, NULL, state->sheet, 0, 0);
+		if (state->name.position) {
+			CellRef tmp;
+			if (cellref_a1_get (&tmp, state->name.position, &pos.eval)) {
+				pos.eval.col = tmp.col;
+				pos.eval.row = tmp.row;
+			}
+		}
+		if (!expr_name_create (&pos, state->name.name,
 				       state->name.value, &perr))
 			g_warning (perr.message);
 		parse_error_free (&perr);
@@ -1403,25 +1583,44 @@ xml_sax_finish_parse_wb_names_name (XMLSaxParseState *state)
 		g_warning ("Can't process named expression '%s'. Ignoring!", state->name.name);
 	}
 
-	g_free (state->name.name);
+	if (state->name.position) {
+		g_free (state->name.position);
+		state->name.position = NULL;
+	}
+
 	g_free (state->name.value);
-	state->name.name = NULL;
 	state->name.value = NULL;
+	g_free (state->name.name);
+	state->name.name = NULL;
 }
 
 static void
 xml_sax_finish_parse_sheet_names_name (XMLSaxParseState *state)
 {
 	ParseError  perr;
+	ParsePos    pos;
 	
 	g_return_if_fail (state->name.name != NULL);
 	g_return_if_fail (state->name.value != NULL);
 
-	if (!expr_name_create (NULL, state->sheet, state->name.name,
+	parse_pos_init (&pos, NULL, state->sheet, 0, 0);
+	if (state->name.position) {
+		CellRef tmp;
+		if (cellref_a1_get (&tmp, state->name.position, &pos.eval)) {
+			pos.eval.col = tmp.col;
+			pos.eval.row = tmp.row;
+		}
+	}
+
+	if (!expr_name_create (&pos, state->name.name,
 			       state->name.value, &perr))
 		g_warning (perr.message);
 	parse_error_free (&perr);
-			  
+
+	if (state->name.position) {
+		g_free (state->name.position);
+		state->name.position = NULL;
+	}
 	g_free (state->name.name);
 	g_free (state->name.value);
 	state->name.name = NULL;
@@ -1445,6 +1644,11 @@ xml_sax_name (XMLSaxParseState *state)
 		g_return_if_fail (state->name.value == NULL);
 		state->name.value = g_strndup (content, len);
 		break;
+	case STATE_SHEET_NAMES_NAME_POSITION:
+	case STATE_NAMES_NAME_POSITION:
+		g_return_if_fail (state->name.position == NULL);
+		state->name.position = g_strndup (content, len);
+		break;
 	default:
 		return;
 	}
@@ -1579,6 +1783,7 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 	case STATE_SHEET_NAMES_NAME:
 		if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME_NAME)) {
 		} else if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME_VALUE)) {
+		} else if (xml_sax_switch_state (state, name, STATE_SHEET_NAMES_NAME_POSITION)) {
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
@@ -1597,6 +1802,8 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 
 	case STATE_SHEET_PRINTINFO :
 		if (xml_sax_switch_state (state, name, STATE_PRINT_MARGINS)) {
+		} else if (xml_sax_switch_state (state, name, STATE_PRINT_SCALE)) {
+			xml_sax_print_scale (state, attrs);
 		} else if (xml_sax_switch_state (state, name, STATE_PRINT_VCENTER)) {
 		} else if (xml_sax_switch_state (state, name, STATE_PRINT_HCENTER)) {
 		} else if (xml_sax_switch_state (state, name, STATE_PRINT_GRID)) {
@@ -1620,8 +1827,7 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		    xml_sax_switch_state (state, name, STATE_PRINT_MARGIN_BOTTOM) ||
 		    xml_sax_switch_state (state, name, STATE_PRINT_MARGIN_LEFT) ||
 		    xml_sax_switch_state (state, name, STATE_PRINT_MARGIN_RIGHT) ||
-		    xml_sax_switch_state (state, name,
-				     STATE_PRINT_MARGIN_HEADER) ||
+		    xml_sax_switch_state (state, name, STATE_PRINT_MARGIN_HEADER) ||
 		    xml_sax_switch_state (state, name, STATE_PRINT_MARGIN_FOOTER)) {
 			xml_sax_print_margins (state, attrs);
 		} else
@@ -1646,6 +1852,8 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		if (xml_sax_switch_state (state, name, STATE_STYLE_FONT))
 			xml_sax_styleregion_font (state, attrs);
 		else if (xml_sax_switch_state (state, name, STATE_STYLE_BORDER)) {
+		} else if (xml_sax_switch_state (state, name, STATE_STYLE_VALIDATION)) {
+			xml_sax_validation (state, attrs);
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
@@ -1661,7 +1869,31 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 		else
 			xml_sax_unknown_state (state, name);
 		break;
+		
+	case STATE_STYLE_VALIDATION :
+		if (xml_sax_switch_state (state, name, STATE_STYLE_CONDITION_CHAIN));
+		else
+			xml_sax_unknown_state (state, name);
+		break;
+		
+	case STATE_STYLE_CONDITION_CHAIN :
+		if (xml_sax_switch_state (state, name, STATE_STYLE_CONDITION))
+			xml_sax_style_condition (state, attrs);
+		else
+			xml_sax_unknown_state (state, name);
+		break;
 
+	case STATE_STYLE_CONDITION :
+		if (xml_sax_switch_state (state, name, STATE_STYLE_CONDITION_EXPR))
+			xml_sax_style_condition_expr (state, attrs);
+		else if (xml_sax_switch_state (state, name, STATE_STYLE_CONDITION_CONSTRAINT))
+			xml_sax_style_condition_constraint (state, attrs);
+		else if (xml_sax_switch_state (state, name, STATE_STYLE_CONDITION_FLAGS))
+			xml_sax_style_condition_flags (state, attrs);
+		else
+			xml_sax_unknown_state (state, name);
+		break;
+		
 	case STATE_SHEET_COLS :
 		if (xml_sax_switch_state (state, name, STATE_COL))
 			xml_sax_colrow (state, attrs, TRUE);
@@ -1722,6 +1954,7 @@ xml_sax_start_element (XMLSaxParseState *state, CHAR const *name, CHAR const **a
 	case STATE_NAMES_NAME:
 		if (xml_sax_switch_state (state, name, STATE_NAMES_NAME_NAME)) {
 		} else if (xml_sax_switch_state (state, name, STATE_NAMES_NAME_VALUE)) {
+		} else if (xml_sax_switch_state (state, name, STATE_NAMES_NAME_POSITION)) {
 		} else
 			xml_sax_unknown_state (state, name);
 		break;
@@ -1803,6 +2036,7 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		break;
 	case STATE_SHEET_NAMES_NAME_NAME :
 	case STATE_SHEET_NAMES_NAME_VALUE :
+	case STATE_SHEET_NAMES_NAME_POSITION :
 		xml_sax_name (state);
 		g_string_truncate (state->content, 0);
 		break;
@@ -1820,6 +2054,10 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		g_string_truncate (state->content, 0);
 		break;
 
+	case STATE_STYLE_VALIDATION :
+		xml_sax_validation_end (state);
+		break;
+		
 	case STATE_CELL :
 		if (state->cell.row >= 0 || state->cell.col >= 0)
 			xml_sax_cell_content (state);
@@ -1840,6 +2078,7 @@ xml_sax_end_element (XMLSaxParseState *state, const CHAR *name)
 		break;
 	case STATE_NAMES_NAME_NAME :
 	case STATE_NAMES_NAME_VALUE :
+	case STATE_NAMES_NAME_POSITION :
 		xml_sax_name (state);
 		g_string_truncate (state->content, 0);
 		break;
@@ -1867,6 +2106,7 @@ xml_sax_characters (XMLSaxParseState *state, const CHAR *chars, int len)
 	case STATE_SHEET_ZOOM :
 	case STATE_SHEET_NAMES_NAME_NAME :
 	case STATE_SHEET_NAMES_NAME_VALUE :
+	case STATE_SHEET_NAMES_NAME_POSITION :
 	case STATE_PRINT_MARGIN_TOP :
 	case STATE_PRINT_MARGIN_BOTTOM :
 	case STATE_PRINT_MARGIN_LEFT :
@@ -1882,6 +2122,7 @@ xml_sax_characters (XMLSaxParseState *state, const CHAR *chars, int len)
 	case STATE_SHEET_MERGE :
 	case STATE_NAMES_NAME_NAME :
 	case STATE_NAMES_NAME_VALUE :
+	case STATE_NAMES_NAME_POSITION :
 		while (len-- > 0)
 			g_string_append_c (state->content, *chars++);
 
@@ -1912,7 +2153,7 @@ xml_sax_start_document (XMLSaxParseState *state)
 	state->attribute.type = -1;
 	state->attributes = NULL;
 
-	state->name.name = state->name.value = NULL;
+	state->name.name = state->name.value = state->name.position = NULL;
 
 	state->style_range_init = FALSE;
 	state->style = NULL;
