@@ -6,10 +6,12 @@
  *   Morten Welinder (terra@diku.dk)
  */
 #include <config.h>
-#include <gnome.h>
+#include <glib.h>
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
 #include <glade/glade.h>
 #include "gnumeric.h"
-#include "gnumeric-util.h"
+#include "gui-util.h"
 #include "dialogs.h"
 #include "search.h"
 #include "widgets/gnumeric-expr-entry.h"
@@ -22,7 +24,7 @@ typedef struct {
 	GladeXML *gui;
 	GnomeDialog *dialog;
 	GnumericExprEntry *rangetext;
-	SearchReplaceDialogCallback cb;
+	SearchDialogCallback cb;
 } DialogState;
 
 static const char *error_group[] = {
@@ -47,6 +49,12 @@ static const char *scope_group[] = {
 	0
 };
 
+static const char *direction_group[] = {
+	"row_major",
+	"column_major",
+	0
+};
+
 static gboolean
 is_checked (GladeXML *gui, const char *name)
 {
@@ -54,6 +62,13 @@ is_checked (GladeXML *gui, const char *name)
 	return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
 }
 
+static void
+set_checked (GladeXML *gui, const char *name, gboolean checked)
+{
+	GtkWidget *w = glade_xml_get_widget (gui, name);
+	return gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), checked);
+}
+
 static char *
 get_text (GladeXML *gui, const char *name)
 {
@@ -67,7 +82,7 @@ ok_clicked (GtkWidget *widget, DialogState *dd)
 	GladeXML *gui = dd->gui;
 	GnomeDialog *dialog = dd->dialog;
 	WorkbookControlGUI *wbcg = dd->wbcg;
-	SearchReplaceDialogCallback cb = dd->cb;
+	SearchDialogCallback cb = dd->cb;
 	SearchReplace *sr;
 	char *err;
 	int i;
@@ -86,28 +101,31 @@ ok_clicked (GtkWidget *widget, DialogState *dd)
 		gtk_entry_get_text (GTK_ENTRY (dd->rangetext)));
 
 	sr->query = is_checked (gui, "query");
-	sr->ignore_case = is_checked (gui, "ignore_case");
 	sr->preserve_case = is_checked (gui, "preserve_case");
+	sr->ignore_case = is_checked (gui, "ignore_case");
 	sr->match_words = is_checked (gui, "match_words");
 
-	sr->replace_strings = is_checked (gui, "replace_string");
-	sr->replace_other_values = is_checked (gui, "replace_other");
-	sr->replace_expressions = is_checked (gui, "replace_expr");
-	sr->replace_comments = is_checked (gui, "replace_comments");
+	sr->search_strings = is_checked (gui, "search_string");
+	sr->search_other_values = is_checked (gui, "search_other");
+	sr->search_expressions = is_checked (gui, "search_expr");
+	sr->search_comments = is_checked (gui, "search_comments");
 
 	i = gnumeric_glade_group_value (gui, error_group);
 	sr->error_behaviour = (i == -1) ? SRE_fail : (SearchReplaceError)i;
 
-	err = search_replace_verify (sr);
+	i = gnumeric_glade_group_value (gui, direction_group);
+	sr->by_row = (i == 0);
+
+	err = search_replace_verify (sr, TRUE);
 	if (err) {
 		gnumeric_notice (wbcg, GNOME_MESSAGE_BOX_ERROR, err);
 		g_free (err);
 		search_replace_free (sr);
 		return;
-	} else if (!sr->replace_strings &&
-		   !sr->replace_other_values &&
-		   !sr->replace_expressions &&
-		   !sr->replace_comments) {
+	} else if (!sr->search_strings &&
+		   !sr->search_other_values &&
+		   !sr->search_expressions &&
+		   !sr->search_comments) {
 		gnumeric_notice (wbcg, GNOME_MESSAGE_BOX_ERROR,
 				 _("You must select some cell types to search."));
 		search_replace_free (sr);
@@ -144,7 +162,7 @@ dialog_destroy (GtkWidget *widget, DialogState *dd)
 static void
 set_focus (GtkWidget *widget, GtkWidget *focus_widget, DialogState *dd)
 {
-	if (GNUMERIC_IS_EXPR_ENTRY (focus_widget))
+	if (IS_GNUMERIC_EXPR_ENTRY (focus_widget))
 		wbcg_set_entry (dd->wbcg,
 				    GNUMERIC_EXPR_ENTRY (focus_widget));
 	else
@@ -161,17 +179,19 @@ range_focused (GtkWidget *widget, GdkEventFocus   *event, DialogState *dd)
 }
 
 static void
-non_model_dialog (WorkbookControlGUI *wbcg, GnomeDialog *dialog)
+non_model_dialog (WorkbookControlGUI *wbcg,
+		  GnomeDialog *dialog,
+		  const char *key)
 {
-	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (dialog), SEARCH_REPLACE_KEY);
-	
+	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (dialog), key);
+
 	gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 
 void
 dialog_search_replace (WorkbookControlGUI *wbcg,
-		       SearchReplaceDialogCallback cb)
+		       SearchDialogCallback cb)
 {
 	GladeXML *gui;
 	GnomeDialog *dialog;
@@ -201,15 +221,15 @@ dialog_search_replace (WorkbookControlGUI *wbcg,
 
 	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
 
-	dd->rangetext = GNUMERIC_EXPR_ENTRY (gnumeric_expr_entry_new ());
+	dd->rangetext = GNUMERIC_EXPR_ENTRY (gnumeric_expr_entry_new (wbcg));
 	gnumeric_expr_entry_set_flags (
-		GNUMERIC_EXPR_ENTRY (dd->rangetext),
+		dd->rangetext,
 		GNUM_EE_SHEET_OPTIONAL, GNUM_EE_SHEET_OPTIONAL);
-	hbox = GTK_BOX (glade_xml_get_widget (gui, "range-hbox"));
+	hbox = GTK_BOX (glade_xml_get_widget (gui, "range_hbox"));
 	gtk_box_pack_start (hbox, GTK_WIDGET (dd->rangetext),
 			    TRUE, TRUE, 0);
 	gtk_widget_show (GTK_WIDGET (dd->rangetext));
-	
+
 	gtk_signal_connect (GTK_OBJECT (glade_xml_get_widget (gui, "ok_button")),
 			    "clicked",
 			    GTK_SIGNAL_FUNC (ok_clicked),
@@ -232,5 +252,53 @@ dialog_search_replace (WorkbookControlGUI *wbcg,
 				     wb_control_gui_cur_sheet (wbcg));
 	wbcg_edit_attach_guru (wbcg, GTK_WIDGET (dialog));
 
-	non_model_dialog (wbcg, dialog);
+	non_model_dialog (wbcg, dialog, SEARCH_REPLACE_KEY);
 }
+
+int
+dialog_search_replace_query (WorkbookControlGUI *wbcg,
+			     SearchReplace *sr,
+			     const char *location,
+			     const char *old_text,
+			     const char *new_text)
+{
+	GladeXML *gui;
+	GnomeDialog *dialog;
+	int res;
+
+	g_return_val_if_fail (wbcg != NULL, 0);
+
+	gui = gnumeric_glade_xml_new (wbcg, "search-replace.glade");
+        if (gui == NULL)
+                return 0;
+
+	dialog = GNOME_DIALOG (glade_xml_get_widget (gui, "query_dialog"));
+
+	gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (gui, "qd_location")),
+			    location);
+	gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (gui, "qd_old_text")),
+			    old_text);
+	gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (gui, "qd_new_text")),
+			    new_text);
+	set_checked (gui, "qd_query", sr->query);
+
+	gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
+
+	gtk_widget_show_all (dialog->vbox);
+
+	res = gnumeric_dialog_run (wbcg, dialog);
+
+	/* Unless cancel is pressed, propagate the query setting back.  */
+	if (res != -1)
+		sr->query = is_checked (gui, "qd_query");
+
+	if (res != -1)
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	if (res == 2)
+		res = -1;
+
+	return res;
+}
+
+/* ------------------------------------------------------------------------- */
