@@ -21,22 +21,6 @@
  */
 
 #include <config.h>
-#include <gnome.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
-#include <math.h>
-#include <float.h>
-#include <limits.h>
-#include <ctype.h>
-#include <locale.h>
-#include <errno.h>
-#ifdef HAVE_LANGINFO_H
-#    include <langinfo.h>
-#endif
-#ifdef HAVE_IEEEFP_H
-#    include <ieeefp.h>
-#endif
 #include "style-color.h"
 #include "format.h"
 #include "expr.h"
@@ -47,6 +31,21 @@
 #include "datetime.h"
 #include "mathfunc.h"
 #include "str.h"
+#include "number-match.h"
+
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <locale.h>
+#include <ctype.h>
+#include <stdio.h>
+#include <errno.h>
+#include <stdlib.h>
+#ifdef HAVE_LANGINFO_H
+#    include <langinfo.h>
+#endif
+#ifdef HAVE_IEEEFP_H
+#    include <ieeefp.h>
+#endif
 
 /***************************************************************************/
 
@@ -69,7 +68,10 @@ format_get_decimal (void)
 	if (lc == NULL)
 		lc = localeconv ();
 
-	res = lc->mon_decimal_point [0];
+	/* NOTE : Use decimal_point _not_ mon_decimal_point.  strtod uses this
+	 * and we get very confused when they are different (eg ru_RU)
+	 */
+	res = lc->decimal_point[0];
 	return (res != '\0') ? res : '.';
 }
 
@@ -80,7 +82,7 @@ format_get_thousand (void)
 	if (lc == NULL)
 		lc = localeconv ();
 
-	res = lc->mon_thousands_sep [0];
+	res = lc->mon_thousands_sep[0];
 	if (res != '\0')
 		return res;
 
@@ -131,7 +133,7 @@ format_month_before_day (void)
 	static gboolean month_first = TRUE;
 
 	if (!date_order_cached) {
-		char const *ptr = nl_langinfo(D_FMT);
+		char const *ptr = nl_langinfo (D_FMT);
 
 		date_order_cached = TRUE;
 		month_first = TRUE;
@@ -157,7 +159,7 @@ format_month_before_day (void)
 #endif
 }
 
-/* Use comma as the arg seperator unless the decimal point is a
+/* Use comma as the arg separator unless the decimal point is a
  * comma, in which case use a semi-colon
  */
 char
@@ -189,12 +191,6 @@ typedef struct {
 	StyleColor *color;
 } StyleFormatEntry;
 
-struct _StyleFormat {
-	int    ref_count;
-	char  *format;
-        GSList *entries;  /* Of type StyleFormatEntry. */
-};
-
 /*
  * The returned string is newly allocated.
  *
@@ -221,14 +217,14 @@ struct _StyleFormat {
 static int
 append_year (GString *string, const guchar *format, const struct tm *time_split)
 {
-	char temp [5];
+	char temp[5];
 
-	if (tolower (format [1]) != 'y'){
+	if (tolower (format[1]) != 'y'){
 		g_string_append_c (string, 'y');
 		return 1;
 	}
 
-	if (tolower (format [2]) != 'y' || tolower (format [3]) != 'y'){
+	if (tolower (format[2]) != 'y' || tolower (format[3]) != 'y'){
 		sprintf (temp, "%02d", time_split->tm_year % 100);
 		g_string_append (string, temp);
 		return 2;
@@ -247,26 +243,26 @@ append_year (GString *string, const guchar *format, const struct tm *time_split)
 static int
 append_month (GString *string, int n, const struct tm *time_split)
 {
-	char temp [3];
+	char temp[3];
 
 	if (n == 1){
-		sprintf (temp, "%d", time_split->tm_mon+1);
+		sprintf (temp, "%d", time_split->tm_mon + 1);
 		g_string_append (string, temp);
 		return 1;
 	}
 
 	if (n == 2){
-		sprintf (temp, "%02d", time_split->tm_mon+1);
+		sprintf (temp, "%02d", time_split->tm_mon + 1);
 		g_string_append (string, temp);
 		return 2;
 	}
 
 	if (n == 3){
-		g_string_append (string, _(month_short [time_split->tm_mon])+1);
+		g_string_append (string, _(month_short[time_split->tm_mon]) + 1);
 		return 3;
 	}
 
-	g_string_append (string, _(month_long [time_split->tm_mon]));
+	g_string_append (string, _(month_long[time_split->tm_mon]));
 	return 4;
 }
 
@@ -279,20 +275,20 @@ append_day (GString *string, const guchar *format, const struct tm *time_split)
 {
 	char temp[3];
 
-	if (tolower (format [1]) != 'd'){
+	if (tolower (format[1]) != 'd'){
 		sprintf (temp, "%d", time_split->tm_mday);
 		g_string_append (string, temp);
 		return 1;
 	}
 
-	if (tolower (format [2]) != 'd'){
+	if (tolower (format[2]) != 'd'){
 		sprintf (temp, "%02d", time_split->tm_mday);
 		g_string_append (string, temp);
 		return 2;
 	}
 
-	if (tolower (format [3]) != 'd'){
-		g_string_append (string, _(day_short[time_split->tm_wday])+1);
+	if (tolower (format[3]) != 'd'){
+		g_string_append (string, _(day_short[time_split->tm_wday]) + 1);
 		return 3;
 	}
 
@@ -456,6 +452,7 @@ format_compile (StyleFormat *format)
 	int num_entries = 1, counter = 0;
 	GSList *ptr;
 
+	format_match_create (format);
 	for (fmt = format->format; *fmt ; fmt++) {
 		if (NULL == real_start && '[' != *fmt)
 			real_start = fmt;
@@ -472,10 +469,10 @@ format_compile (StyleFormat *format)
 
 			/* Check for conditional */
 			if (*begin == '<') {
-				if (begin [1] == '=') {
+				if (begin[1] == '=') {
 					entry->restriction_type = ',';
 					begin += 2;
-				} else if (begin [1] == '>') {
+				} else if (begin[1] == '>') {
 					entry->restriction_type = '+';
 					begin += 2;
 				} else {
@@ -483,7 +480,7 @@ format_compile (StyleFormat *format)
 					begin++;
 				}
 			} else if (*begin == '>') {
-				if (begin [1] == '=') {
+				if (begin[1] == '=') {
 					entry->restriction_type = '.';
 					begin += 2;
 				} else {
@@ -533,7 +530,7 @@ format_compile (StyleFormat *format)
 		}
 
 		case '/':
-			if (fmt [1] == '?' || isdigit (fmt[1])) {
+			if (fmt[1] == '?' || isdigit (fmt[1])) {
 				entry->has_fraction = TRUE;
 				fmt++;
 			}
@@ -541,7 +538,7 @@ format_compile (StyleFormat *format)
 
 		case 'a': case 'A':
 		case 'p': case 'P':
-			if (tolower (fmt [1]) == 'm')
+			if (tolower (fmt[1]) == 'm')
 				entry->want_am_pm = TRUE;
 			break;
 
@@ -598,12 +595,13 @@ format_destroy (StyleFormat *format)
 	g_slist_foreach (format->entries, &format_entry_dtor, NULL);
 	g_slist_free (format->entries);
 	format->entries = NULL;
+	format_match_release (format);
 }
 
 static struct FormatColor {
 	char       *name;
 	StyleColor *color;
-} format_colors [] = {
+} format_colors[] = {
 	{ N_("black")   },
 	{ N_("blue")    },
 	{ N_("cyan")    },
@@ -620,15 +618,9 @@ format_color_init (void)
 {
 	int i;
 
-	for (i = 0; format_colors [i].name; i++){
-		StyleColor *sc;
-		GdkColor c;
-
-		gdk_color_parse (format_colors [i].name, &c);
-		sc = style_color_new (c.red, c.green, c.blue);
-
-		format_colors [i].color = sc;
-	}
+	for (i = 0; format_colors[i].name; i++)
+		format_colors[i].color =
+			style_color_new_name (format_colors[i].name);
 }
 
 void
@@ -636,8 +628,8 @@ format_color_shutdown (void)
 {
 	int i;
 
-	for (i = 0; format_colors [i].name; i++)
-		style_color_unref (format_colors [i].color);
+	for (i = 0; format_colors[i].name; i++)
+		style_color_unref (format_colors[i].color);
 }
 
 static struct FormatColor const *
@@ -647,12 +639,12 @@ lookup_color_by_name (unsigned char const *str, unsigned char const *end,
 	int i, len;
 
 	len = end - str;
-	for (i = 0; format_colors [i].name; i++) {
-		char const *name = format_colors [i].name;
+	for (i = 0; format_colors[i].name; i++) {
+		char const *name = format_colors[i].name;
 		if (translate)
 			name = _(name);
 
-		if (0 == g_strncasecmp (name, str, len) && name [len] == '\0')
+		if (0 == g_strncasecmp (name, str, len) && name[len] == '\0')
 			return format_colors + i;
 	}
 	return NULL;
@@ -697,8 +689,8 @@ render_number (GString *result,
 	       format_info_t const *info)
 {
 	char thousands_sep;
-	unsigned char num_buf [(DBL_MANT_DIG + DBL_MAX_EXP)*2 + 1];
-	unsigned char *num = num_buf + sizeof(num_buf) - 1;
+	unsigned char num_buf[(DBL_MANT_DIG + DBL_MAX_EXP) * 2 + 1];
+	unsigned char *num = num_buf + sizeof (num_buf) - 1;
 	double frac_part, int_part;
 	int group, zero_count, digit_count = 0;
 
@@ -707,7 +699,7 @@ render_number (GString *result,
 	if (right_allowed >= 0 && !info->has_fraction) {
 		/* Change "rounding" into "truncating".   */
 		/* Note, that we assume number >= 0 here. */
-		gdouble delta = 5 * gpow10 (-right_allowed-1);
+		gdouble delta = 5 * gpow10 (-right_allowed - 1);
 		number += delta;
 	}
 	frac_part = modf (gnumeric_add_epsilon (number), &int_part);
@@ -788,7 +780,7 @@ render_number (GString *result,
 static void
 do_render_number (gdouble number, format_info_t *info, GString *result)
 {
-	char decimal_point [2];
+	char decimal_point[2];
 
 	info->rendered = TRUE;
 
@@ -798,10 +790,10 @@ do_render_number (gdouble number, format_info_t *info, GString *result)
 	 * decimal point
 	 */
 	if (number > 0.0 && number < 1.0 && info->right_allowed == 0 && info->right_optional > 0){
-		decimal_point [0] = format_get_decimal ();
-		decimal_point [1] = 0;
+		decimal_point[0] = format_get_decimal ();
+		decimal_point[1] = 0;
 	} else
-		decimal_point [0] = 0;
+		decimal_point[0] = 0;
 
 #if 0
 	printf ("Rendering: %g with:\n", number);
@@ -906,7 +898,7 @@ find_decimal_char (char const *str)
 
 			/* Escaped char and spacing format */
 		case '\\': case '_':
-			if (*(str+1))
+			if (*(str + 1))
 				str++;
 			break;
 
@@ -990,7 +982,7 @@ format_add_decimal (StyleFormat const *fmt)
 
 	if (strcmp (format_string, "General") == 0) {
 		format_string = "0";
-		pre = format_string+1;
+		pre = format_string + 1;
 		post = pre;
 	} else {
 		pre = find_decimal_char (format_string);
@@ -1013,7 +1005,7 @@ format_add_decimal (StyleFormat const *fmt)
 				++pre;
 			post = pre;
 		} else
-			post = pre+1;
+			post = pre + 1;
 	}
 	res = g_malloc ((pre - format_string + 1) +
 		      1 + /* for the decimal */
@@ -1024,8 +1016,8 @@ format_add_decimal (StyleFormat const *fmt)
 		return NULL;
 
 	strncpy (res, format_string, pre - format_string);
-	res [pre-format_string + 0] = '.';
-	res [pre-format_string + 1] = '0';
+	res[pre-format_string + 0] = '.';
+	res[pre-format_string + 1] = '0';
 	strcpy (res + (pre - format_string) + 2, post);
 
 	return res;
@@ -1107,7 +1099,7 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 			break;
 
 		case '.': {
-			int c = *(format+1);
+			int c = *(format + 1);
 
 			can_render_number = TRUE;
 			if (c && (c != '0' && c != '#' && c != '?'))
@@ -1188,7 +1180,7 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 				int size = 0;
 				int numerator = -1, denominator = -1;
 
-				while (format [size+1] == '?')
+				while (format[size + 1] == '?')
 					++size;
 
 				/* check for explicit denominator */
@@ -1196,18 +1188,18 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 					char *end;
 
 					errno = 0;
-					denominator = strtol (format+1, &end, 10);
-					if (format+1 != end && errno != ERANGE) {
+					denominator = strtol (format + 1, &end, 10);
+					if (format + 1 != end && errno != ERANGE) {
 						format = end;
 						numerator = (int)((number - (int)number) * denominator + 0.5);
 					}
 				} else {
-					static int const powers [3] = { 10, 100, 1000 };
+					static int const powers[3] = { 10, 100, 1000 };
 
 					format += size + 1;
 					if (size > 3)
 						size = 3;
-					stern_brocot (number - (int)number, powers [size-1],
+					stern_brocot (number - (int)number, powers[size - 1],
 						      &numerator, &denominator);
 				}
 
@@ -1220,7 +1212,7 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 							g_string_prepend_c (result, '-');
 					}
 					if (numerator > 0) {
-						char buffer [30];
+						char buffer[30];
 						sprintf (buffer, "%d/%d", numerator, denominator);
 						g_string_append (result, buffer);
 					}
@@ -1274,7 +1266,7 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 			 * assumption is WRONG! but ok until we rewrite the
 			 * format engine.
 			 */
-			if (format [1]) {
+			if (format[1]) {
 				if (can_render_number && !info.rendered)
 					do_render_number (number, &info, result);
 				fill_char = *(++format);
@@ -1293,9 +1285,9 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 			 * This is a problem waiting to happen.
 			 * rewrite.
 			 */
-			for (n = 1; format [1] == 'M' || format [1] == 'm'; format++)
+			for (n = 1; format[1] == 'M' || format[1] == 'm'; format++)
 				n++;
-			if (format [1] == ']')
+			if (format[1] == ']')
 				format++;
 			if (time_display_elapsed) {
 				time_display_elapsed = FALSE;
@@ -1330,9 +1322,9 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 			if (!time_split)
 				time_split = split_time (signed_number);
 
-			for (n = 1; format [1] == 's' || format [1] == 'S'; format++)
+			for (n = 1; format[1] == 's' || format[1] == 'S'; format++)
 				n++;
-			if (format [1] == ']')
+			if (format[1] == ']')
 				format++;
 			if (time_display_elapsed){
 				time_display_elapsed = FALSE;
@@ -1350,17 +1342,23 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 			if (!time_split)
 				time_split = split_time (signed_number);
 
-			for (n = 1; format [1] == 'h' || format [1] == 'H'; format++)
+			for (n = 1; format[1] == 'h' || format[1] == 'H'; format++)
 				n++;
-			if (format [1] == ']')
+			if (format[1] == ']')
 				format++;
 			if (time_display_elapsed){
 				time_display_elapsed = FALSE;
 				ignore_further_elapsed = TRUE;
 				append_hour_elapsed (result, n, time_split, number);
 			} else
+				/* h == hour optionally in 24 hour mode
+				 * h followed by am/pm puts it in 12 hout mode
+				 *
+				 * multiple h eg 'hh' force 12 hour mode.
+				 * NOTE : This is a non-XL extension
+				 */
 				append_hour (result, n, time_split,
-					     entry->want_am_pm);
+					     entry->want_am_pm || (n > 1));
 			hour_seen = TRUE;
 			break;
 		}
@@ -1374,13 +1372,13 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 				format++;
 				if (*format == 'm' || *format == 'M'){
 					g_string_append_c (result, *format);
-					if (*(format+1) == '/')
+					if (*(format + 1) == '/')
 						format++;
 				}
 			} else {
-				if (*(format+1) == 'm' || *(format+1) == 'M')
+				if (*(format + 1) == 'm' || *(format + 1) == 'M')
 					format++;
-				if (*(format+1) == '/')
+				if (*(format + 1) == '/')
 					format++;
 			}
 			break;
@@ -1390,17 +1388,20 @@ format_number (gdouble number, int col_width, StyleFormatEntry const *entry)
 				time_split = split_time (signed_number);
 			if (time_split->tm_hour >= 12){
 				g_string_append_c (result, *format);
-				if (*(format+1) == 'm' || *(format+1) == 'M'){
+				if (*(format + 1) == 'm' || *(format + 1) == 'M'){
 					format++;
 					g_string_append_c (result, *format);
 				}
 			} else {
-				if (*(format+1) == 'm' || *(format+1) == 'M')
+				if (*(format + 1) == 'm' || *(format + 1) == 'M')
 					format++;
 			}
 			break;
 
 		default:
+			/* TODO : After release check this.
+			 * shouldn't we tack on the explicit characters here ?
+			 */
 			break;
 		}
 		format++;
@@ -1585,7 +1586,7 @@ format_value (StyleFormat *format, const Value *value, StyleColor **color,
 			entry = (StyleFormatEntry const *)(list->data);
 
 			/* Empty formats should be ignored */
-			if (entry->format [0] == '\0')
+			if (entry->format[0] == '\0')
 				return g_strdup ("");
 
 			if (color && entry->color != NULL)
@@ -1685,7 +1686,7 @@ translate_format_color (GString *res, char const *ptr, gboolean translate_to_en)
 	color = lookup_color_by_name (ptr, end, translate_to_en);
 	if (color != NULL) {
 		g_string_append (res, translate_to_en
-				 ?  _(color->name)
+				 ? _(color->name)
 				 : color->name);
 		g_string_append_c (res, ']');
 		return end;
@@ -1724,7 +1725,7 @@ style_format_delocalize (const char *descriptor_string)
 					    break;
 
 				case '\\' : g_string_append_c (res, '\\');
-					    if (ptr [1] != '\0') {
+					    if (ptr[1] != '\0') {
 						    g_string_append_c (res, ptr[1]);
 						    ++ptr;
 					    }
@@ -1768,7 +1769,10 @@ style_format_new_XL (char const *descriptor_string, gboolean delocalize)
 		format = g_new0 (StyleFormat, 1);
 		format->format = g_strdup (descriptor_string);
 		format->entries = NULL;
-		format_compile (format);
+		format->regexp_str = NULL;
+		format->match_tags = NULL;
+		if (strcmp ("General", format->format))
+			format_compile (format);
 		g_hash_table_insert (style_format_hash, format->format, format);
 	}
 	format->ref_count++;
@@ -1832,7 +1836,7 @@ style_format_str_as_XL (char const *ptr, gboolean localized)
 			    break;
 
 		case '\\' : g_string_append_c (res, '\\');
-			    if (ptr [1] != '\0') {
+			    if (ptr[1] != '\0') {
 				    g_string_append_c (res, ptr[1]);
 				    ++ptr;
 			    }
