@@ -988,6 +988,7 @@ static char *help_log = {
 	   "@DESCRIPTION="
 	   "LOG computes the logarithm of @x in the given base @base.  "
 	   "If no @base is given LOG returns the logarithm in base 10. "
+	   "@base must be > 0. and can not equal 1. "
 	   "This function is Excel compatible. "
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1008,7 +1009,7 @@ gnumeric_log (FunctionEvalInfo *ei, Value **argv)
 	        base = 10;
 	else {
 	        base = value_get_as_float (argv[1]);
-		if (base <= 1)
+		if (base == 1. || base <= 0.)
 			return value_new_error (ei->pos, gnumeric_err_NUM);
 	}
 
@@ -2813,7 +2814,7 @@ gnumeric_subtotal (FunctionEvalInfo *ei, GList *expr_node_list)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei->pos, tree, EVAL_STRICT);
+	val = expr_eval (tree, ei->pos, EVAL_STRICT);
 	if (!val) return NULL;
 	if (!VALUE_IS_NUMBER (val)) {
 		value_release (val);
@@ -2908,7 +2909,7 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei->pos, tree, EVAL_STRICT);
+	val = expr_eval (tree, ei->pos, EVAL_STRICT);
 	if (!val) return NULL;
 	if (!VALUE_IS_NUMBER (val)) {
 		value_release (val);
@@ -2924,7 +2925,7 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei->pos, tree, EVAL_STRICT);
+	val = expr_eval (tree, ei->pos, EVAL_STRICT);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER (val)) {
 		value_release (val);
@@ -2943,7 +2944,7 @@ gnumeric_seriessum (FunctionEvalInfo *ei, GList *nodes)
 	if (tree == NULL)
 		return value_new_error (ei->pos, gnumeric_err_NUM);
 
-	val = eval_expr (ei->pos, tree, EVAL_STRICT);
+	val = expr_eval (tree, ei->pos, EVAL_STRICT);
 	if (!val) return NULL;
 	if (! VALUE_IS_NUMBER (val)) {
 		value_release (val);
@@ -3271,8 +3272,8 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GList *args)
 
 	for (l = args, i = 0; l; l = l->next, i++) {
 		int thissizex, thissizey, x, y;
-		ExprTree *tree = l->data;
-		Value    *val = eval_expr (ei->pos, tree,
+		ExprTree const *expr = l->data;
+		Value    *val = expr_eval (expr, ei->pos,
 					   EVAL_PERMIT_NON_SCALAR | EVAL_PERMIT_EMPTY);
 
 		thissizex = value_area_get_width (ei->pos, val);
