@@ -12,7 +12,7 @@
 #include "mstyle.h"
 #include "style-border.h"
 #include "style-color.h"
-#include "style-condition.h"
+#include "validation.h"
 #include "pattern.h"
 #include "format.h"
 #include "main.h"
@@ -58,10 +58,8 @@ typedef struct {
 		gboolean         content_locked;
 		gboolean         content_hidden;
 
-		StyleCondition  *validation;
-		ValidationStyle  validation_style;
-		String          *validation_msg;
-		
+		Validation      *validation;
+
 		/* Convenience members */
 		gpointer         any_pointer;
 		gboolean         any_boolean;
@@ -90,8 +88,7 @@ struct _MStyle {
 
 #define MSTYLE_ANY_POINTER           MSTYLE_FONT_NAME: \
 				case MSTYLE_FORMAT: \
-				case MSTYLE_VALIDATION: \
-				case MSTYLE_VALIDATION_MSG
+				case MSTYLE_VALIDATION
 
 #define MSTYLE_ANY_BOOLEAN           MSTYLE_FONT_BOLD: \
 				case MSTYLE_FONT_ITALIC: \
@@ -133,10 +130,10 @@ const char *mstyle_names[MSTYLE_ELEMENT_MAX] = {
 	"Align.h",
 	"Indent",
 	"Orientation",
-	"WrapText"
+	"WrapText",
+	"Content.Locked",
+	"Content.Hidden",
 	"Validation"
-	"Validation.Style"
-	"Validation.Msg"
 };
 
 guint
@@ -188,9 +185,6 @@ mstyle_hash (gconstpointer st)
 		case MSTYLE_FONT_UNDERLINE:
 			hash = hash ^ e->u.font.underline;
 			break;
-		case MSTYLE_VALIDATION_STYLE:
-			hash = hash ^ e->u.validation_style;
-			break;
 		default:
 			g_warning ("Unimplemented hash item");
 			break;
@@ -279,9 +273,9 @@ mstyle_element_dump (const MStyleElement *e)
 	case MSTYLE_PATTERN :
 		g_string_sprintf (ans, "pattern %d", e->u.pattern);
 		break;
-
-	case MSTYLE_VALIDATION_MSG:
-		g_string_sprintf (ans, "validation msg '%s'", e->u.validation_msg->str);
+		
+	case MSTYLE_VALIDATION :
+		g_string_sprintf (ans, "validation ref_count '%d'", e->u.validation->ref_count);
 		break;
 
 	default:
@@ -377,14 +371,6 @@ mstyle_element_equal (MStyleElement const *a,
 		if (a->u.validation == b->u.validation)
 			return TRUE;
 		break;
-	case MSTYLE_VALIDATION_STYLE:
-		if (a->u.validation_style == b->u.validation_style)
-			return TRUE;
-		break;
-	case MSTYLE_VALIDATION_MSG:
-		if (a->u.validation_msg == b->u.validation_msg)
-			return TRUE;
-		break;
 	default:
 		return TRUE;
 	}
@@ -414,7 +400,9 @@ mstyle_elements_equal (const MStyleElement *a,
 	for (i = 1; i < MSTYLE_ELEMENT_MAX; i++) {
 		/* Elements in the same position should have the same types */
 		if (a[i].type != b[i].type) {
-			g_warning ("%s mismatched types\n", mstyle_names[i]);
+			if (a[i].type != MSTYLE_ELEMENT_UNSET &&
+			    b[i].type != MSTYLE_ELEMENT_UNSET)
+				g_warning ("%s mismatched types\n", mstyle_names[i]);
 			return FALSE;
 		}
 
@@ -443,11 +431,7 @@ mstyle_element_ref (const MStyleElement *e)
 		break;
 	case MSTYLE_VALIDATION:
 		if (e->u.validation)
-			style_condition_ref (e->u.validation);
-		break;
-	case MSTYLE_VALIDATION_MSG:
-		if (e->u.validation_msg)
-			string_ref (e->u.validation_msg);
+			validation_ref (e->u.validation);
 		break;
 	default:
 		break;
@@ -473,11 +457,7 @@ mstyle_element_unref (MStyleElement e)
 		break;
 	case MSTYLE_VALIDATION:
 		if (e.u.validation)
-			style_condition_unref (e.u.validation);
-		break;
-	case MSTYLE_VALIDATION_MSG:
-		if (e.u.validation_msg)
-			string_unref (e.u.validation_msg);
+			validation_unref (e.u.validation);
 		break;
 	default:
 		break;
@@ -636,10 +616,6 @@ mstyle_new_default (void)
 	mstyle_set_color       (mstyle, MSTYLE_COLOR_PATTERN,
 				style_color_black ());
 
-	mstyle_set_validation       (mstyle, NULL);
-	mstyle_set_validation_style (mstyle, VALIDATION_STYLE_NONE);
-	mstyle_set_validation_msg   (mstyle, NULL);
-	
 	/* To negate borders */
 	mstyle_set_border      (mstyle, MSTYLE_BORDER_TOP,
 				style_border_ref (style_border_none ()));
@@ -1219,17 +1195,18 @@ mstyle_get_content_hidden (const MStyle *style)
 }
 
 void
-mstyle_set_validation (MStyle *style, StyleCondition *sc)
+mstyle_set_validation (MStyle *style, Validation *v)
 {
 	g_return_if_fail (style != NULL);
-
+	g_return_if_fail (v != NULL);
+	
 	mstyle_element_unref (style->elements[MSTYLE_VALIDATION]);
 	style->elements[MSTYLE_VALIDATION].type = MSTYLE_VALIDATION;
-	style->elements[MSTYLE_VALIDATION].u.validation = sc;
+	style->elements[MSTYLE_VALIDATION].u.validation = v;
 	
 }
 
-StyleCondition *
+Validation *
 mstyle_get_validation (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_VALIDATION), NULL);
@@ -1237,42 +1214,6 @@ mstyle_get_validation (const MStyle *style)
 	return style->elements[MSTYLE_VALIDATION].u.validation;
 }
 
-void
-mstyle_set_validation_style (MStyle *style, ValidationStyle vs)
-{
-	g_return_if_fail (style != NULL);
-	
-	style->elements[MSTYLE_VALIDATION_STYLE].type = MSTYLE_VALIDATION_STYLE;
-	style->elements[MSTYLE_VALIDATION_STYLE].u.validation_style = vs;
-}
-
-ValidationStyle
-mstyle_get_validation_style (const MStyle *style)
-{
-	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_VALIDATION_STYLE),
-			      VALIDATION_STYLE_NONE);
-
-	return style->elements[MSTYLE_VALIDATION_STYLE].u.validation_style;
-}
-
-void
-mstyle_set_validation_msg (MStyle *style, const char *msg)
-{
-	g_return_if_fail (style != NULL);
-
-	mstyle_element_unref (style->elements[MSTYLE_VALIDATION_MSG]);
-	style->elements[MSTYLE_VALIDATION_MSG].type = MSTYLE_VALIDATION_MSG;
-	style->elements[MSTYLE_VALIDATION_MSG].u.validation_msg = msg ? string_get (msg) : NULL;
-}
-
-const char *
-mstyle_get_validation_msg (const MStyle *style)
-{
-	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_VALIDATION_MSG), NULL);
-
-	return style->elements[MSTYLE_VALIDATION_MSG].u.validation_msg->str;
-}
-
 gboolean
 mstyle_visible_in_blank (const MStyle *st)
 {
