@@ -163,7 +163,8 @@ print_sheet_objects (PrintJobInfo const *pj, Sheet const *sheet, Range *range,
 		double obj_base_x = 0.0, obj_base_y = 0.0;
 
 		/* First check if we need to print this object */
-		if (!range_overlap (range, &so->anchor.cell_bound))
+		if (!so->is_visible ||
+		    !range_overlap (range, &so->anchor.cell_bound))
 			continue;
 
 		sheet_object_position_pts_get (so, coords);
