@@ -324,18 +324,37 @@ calculate_replacement (SearchReplace *sr, const char *src, const regmatch_t *pm)
 gboolean
 search_match_string (SearchReplace *sr, const char *src)
 {
-	int ret;
+	int flags = 0;
 
 	g_return_val_if_fail (sr && sr->comp_search, FALSE);
 
-	ret = regexec (sr->comp_search, src, 0, 0, REG_NOSUB);
+	while (1) {
+		regmatch_t match;
+		int ret = regexec (sr->comp_search, src, 1, &match, flags);
 
-	switch (ret) {
-	case 0: return TRUE;
-	case REG_NOMATCH: return FALSE;
-	default:
-		g_error ("Unexpect error code from regexec: %d.", ret);
-		return FALSE;
+		switch (ret) {
+		case 0:
+			if (!sr->match_words)
+				return TRUE;
+
+			if (match_is_word (sr, src, &match, (flags & REG_NOTBOL) != 0))
+				return TRUE;
+
+			/*
+			 * We had a match, but it's not a word.  Pretend we saw
+			 * a one-character match and continue after that.
+			 */
+			flags |= REG_NOTBOL;
+			src += match.rm_so + 1;
+			break;
+
+		case REG_NOMATCH:
+			return FALSE;
+
+		default:
+			g_error ("Unexpected error code from regexec: %d.", ret);
+			return FALSE;
+		}
 	}
 }
 
@@ -503,6 +522,65 @@ search_collect_cells (SearchReplace *sr, Sheet *sheet)
 	return cells;
 }
 
+void
+search_collect_cells_free (GPtrArray *cells)
+{
+	unsigned i;
+
+	for (i = 0; i < cells->len; i++)
+		g_free (g_ptr_array_index (cells, i));
+	g_ptr_array_free (cells, TRUE);
+}
+
+/* ------------------------------------------------------------------------- */
+/*
+ * sr: The search spec.
+ * cells: An array of EvalPos*, presumably a result of search_collect_cells.
+ *
+ * Returns an array of SearchFilterResult*s, which the caller must free.
+ */
+
+GPtrArray *
+search_filter_matching (SearchReplace *sr, const GPtrArray *cells)
+{
+	unsigned i;
+	GPtrArray *result = g_ptr_array_new ();
+
+	for (i = 0; i < cells->len; i++) {
+		SearchReplaceCellResult cell_res;
+		SearchReplaceCommentResult comment_res;
+		gboolean found;
+		const EvalPos *ep = g_ptr_array_index (cells, i);
+
+		found = search_replace_cell (sr, ep, FALSE, &cell_res);
+		g_free (cell_res.old_text);
+		if (found) {
+			SearchFilterResult *item = g_new (SearchFilterResult, 1);
+			item->ep = *ep;
+			item->cell = cell_res.cell;
+			g_ptr_array_add (result, item);
+		}
+
+		if (search_replace_comment (sr, ep, FALSE, &comment_res)) {
+			SearchFilterResult *item = g_new (SearchFilterResult, 1);
+			item->ep = *ep;
+			item->comment = comment_res.comment;
+			g_ptr_array_add (result, item);
+		}
+	}
+
+	return result;
+}
+
+void
+search_filter_matching_free (GPtrArray *matches)
+{
+	unsigned i;
+	for (i = 0; i < matches->len; i++)
+		g_free (g_ptr_array_index (matches, i));
+	g_ptr_array_free (matches, TRUE);	
+}
+
 /* ------------------------------------------------------------------------- */
 
 gboolean
