@@ -7,12 +7,6 @@
  *   Miguel de Icaza (miguel@kernel.org)
  */
 #include <config.h>
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <gdk/gdkkeysyms.h>
-#include <libgnomeprint/gnome-print.h>
-#include <math.h>
 #include "gnumeric.h"
 #include "sheet-control-gui.h"
 #include "gnumeric-canvas.h"
@@ -22,6 +16,9 @@
 #include "sheet-object-graphic.h"
 #include "sheet-object-impl.h"
 
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-i18n.h>
+#include <gdk/gdkkeysyms.h>
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-colors.h>
 #include <gal/widgets/widget-color-combo.h>
@@ -178,6 +175,11 @@ sheet_object_graphic_update_bounds (SheetObject *so, GtkObject *view,
 			       "points", points,
 			       NULL);
 	gnome_canvas_points_free (points);
+
+	if (so->is_visible)
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (view));
+	else
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (view));
 }
 
 static gboolean
@@ -370,7 +372,6 @@ cb_dialog_graphic_clicked (GnomeDialog *dialog, int button,
 			   DialogGraphicData *data)
 {
 	SheetObject *so = SHEET_OBJECT (data->sog);
-	GdkColor *gdk;
 
 	switch (button) {
 	case 0: /* Ok */
@@ -378,10 +379,8 @@ cb_dialog_graphic_clicked (GnomeDialog *dialog, int button,
 		sheet_object_graphic_width_set (data->sog,
 				GTK_ADJUSTMENT (data->adj_width)->value);
 
-		gdk = color_combo_get_color (
-			COLOR_COMBO (data->fill_color_combo));
 		sheet_object_graphic_fill_color_set (so,
-			style_color_new (gdk->red, gdk->green, gdk->blue));
+			color_combo_get_style_color (data->fill_color_combo));
 
 		if (data->sog->type == SHEET_OBJECT_ARROW)
 			sheet_object_graphic_abc_set (data->sog, 
@@ -459,15 +458,15 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControlGUI *scg)
 
 	label = gtk_label_new (_("Color"));
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
-	label = gtk_label_new (_("Border width"));
+	label = gtk_label_new (_("Line Width"));
 	gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
 
 	if (sog->type == SHEET_OBJECT_ARROW) {
-		label = gtk_label_new (_("Arrow shape a"));
+		label = gtk_label_new (_("Arrow tip"));
 		gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2,3);
-		label = gtk_label_new (_("Arrow shape b"));
+		label = gtk_label_new (_("Arrow length"));
 		gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3,4);
-		label = gtk_label_new (_("Arrow shape c"));
+		label = gtk_label_new (_("Arrow width"));
 		gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 4,5);
 	}
 
@@ -680,6 +679,11 @@ sheet_object_filled_update_bounds (SheetObject *so, GtkObject *view,
 		"y1", MIN (coords [1], coords [3]),
 		"y2", MAX (coords [1], coords [3]),
 		NULL);
+
+	if (so->is_visible)
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (view));
+	else
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (view));
 }
 
 static GtkObject *
@@ -787,7 +791,6 @@ cb_dialog_filled_clicked (GnomeDialog *dialog, int button,
 {
 	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (data->sof);
 	SheetObject *so = SHEET_OBJECT (data->sof);
-	GdkColor *gdk;
 		
 	switch (button) {
 	case 0: /* Ok */
@@ -795,15 +798,10 @@ cb_dialog_filled_clicked (GnomeDialog *dialog, int button,
 		sheet_object_graphic_width_set (sog, 
 			GTK_ADJUSTMENT (data->adj_width)->value);
 
-		gdk = color_combo_get_color (
-			COLOR_COMBO (data->fill_color_combo));
 		sheet_object_graphic_fill_color_set (so,
-			style_color_new (gdk->red, gdk->green, gdk->blue));
-
-		gdk = color_combo_get_color (
-			COLOR_COMBO (data->outline_color_combo));
+			color_combo_get_style_color (data->fill_color_combo));
 		sheet_object_filled_outline_color_set (so,
-			style_color_new (gdk->red, gdk->green, gdk->blue));
+			color_combo_get_style_color (data->outline_color_combo));
 
 		if (button == 0)
 			gnome_dialog_close (dialog);
