@@ -39,8 +39,9 @@ void gnumeric_expr_entry_get_rangesel	(GnumericExprEntry *expr_entry,
 void gnumeric_expr_entry_rangesel_stopped (GnumericExprEntry *expr_entry,
 					   gboolean clear_string);
 
-void	 gnumeric_expr_entry_set_rangesel_from_text (GnumericExprEntry *expr_entry,
-						     char *text);
+void	 gnumeric_expr_entry_clear 		     (GnumericExprEntry *gee);
+void	 gnumeric_expr_entry_set_rangesel_from_dep   (GnumericExprEntry *gee,
+						      Dependent const *dep);
 gboolean gnumeric_expr_entry_set_rangesel_from_range (GnumericExprEntry *expr_entry,
 						      Range const *r, Sheet *sheet, int pos);
 
